@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::fmt;
 
 use unicode_width::UnicodeWidthStr;
@@ -12,7 +13,7 @@ pub use moentry::MOEntry;
 pub use poentry::POEntry;
 
 /// Provides a function `translated` to represent
-/// if some struct is translated
+/// if an entry struct is translated
 pub trait Translated {
     fn translated(&self) -> bool;
 }
@@ -36,17 +37,18 @@ pub trait MsgidEotMsgctxt {
     fn msgid_eot_msgctxt(&self) -> String;
 }
 
-pub(crate) fn maybe_msgid_msgctxt_eot_split(
-    msgid: &str,
+pub(crate) fn maybe_msgid_msgctxt_eot_split<'a>(
+    msgid: &'a str,
     msgctxt: &Option<String>,
-) -> String {
+) -> Cow<'a, str> {
     if let Some(ctx) = msgctxt {
         let mut ret = String::from(ctx);
+        ret.reserve(msgid.len() + 1);
         ret.push('\u{4}');
         ret.push_str(msgid);
-        ret
+        ret.into()
     } else {
-        msgid.to_string()
+        msgid.into()
     }
 }
 
@@ -179,14 +181,12 @@ pub(crate) fn mo_entry_to_string(
 /// assert!(entry_str.starts_with("msgid \"\"\nmsgstr \"\""));
 /// ```
 pub fn mo_metadata_entry_to_string(entry: &MOEntry) -> String {
-    let mut ret = String::new();
-    ret.push_str(&mo_entry_to_string_with_msgstr_formatter(
+    mo_entry_to_string_with_msgstr_formatter(
         entry,
         78,
         "",
         &metadata_msgstr_formatter,
-    ));
-    ret
+    )
 }
 
 /// Converts a metadata wrapped by a [POEntry] to a string
@@ -283,3 +283,135 @@ impl<'a> fmt::Display for POStringField<'a> {
         write!(f, "{}", ret)
     }
 }
+
+/// A struct to compare two entries.
+///
+/// ```rust
+/// use std::cmp::Ordering;
+/// use rspolib::{POEntry, EntryCmpByOptions};
+///
+/// let mut entry1 = POEntry::from("msgid 1");
+/// let entry2 = POEntry::from("msgid 2");
+///
+/// let compare_by_all_fields = EntryCmpByOptions::new();
+/// let compare_by_msgid_only = EntryCmpByOptions::new()
+///     .by_all(false)
+///     .by_msgid(true);
+///
+/// assert_eq!(entry1.cmp_by(&entry2, &compare_by_msgid_only), Ordering::Less);
+/// assert_eq!(entry2.cmp_by(&entry1, &compare_by_msgid_only), Ordering::Greater);
+///
+/// entry1.msgid = "msgid 2".to_string();
+/// assert_eq!(entry1.cmp_by(&entry2, &compare_by_msgid_only), Ordering::Equal);
+///
+/// entry1.msgstr = Some("msgstr 1".to_string());
+/// assert_eq!(entry1.cmp_by(&entry2, &compare_by_msgid_only), Ordering::Equal);
+/// assert_eq!(entry1.cmp_by(&entry2, &compare_by_all_fields), Ordering::Greater);
+/// ```
+pub struct EntryCmpByOptions {
+    by_msgid: bool,
+    by_msgstr: bool,
+    by_msgctxt: bool,
+    by_obsolete: bool,
+    by_occurrences: bool,
+    by_msgid_plural: bool,
+    by_msgstr_plural: bool,
+    by_flags: bool,
+}
+
+impl EntryCmpByOptions {
+    /// Creates a instance of [EntryCmpByOptions] with comparisons for all fields enabled
+    pub fn new() -> Self {
+        Self {
+            by_msgid: true,
+            by_msgstr: true,
+            by_msgctxt: true,
+            by_obsolete: true,
+            by_occurrences: true,
+            by_msgid_plural: true,
+            by_msgstr_plural: true,
+            by_flags: true,
+        }
+    }
+
+    pub fn by_msgid(mut self, by_msgid: bool) -> Self {
+        self.by_msgid = by_msgid;
+        self
+    }
+
+    pub fn by_msgstr(mut self, by_msgstr: bool) -> Self {
+        self.by_msgstr = by_msgstr;
+        self
+    }
+
+    pub fn by_msgctxt(mut self, by_msgctxt: bool) -> Self {
+        self.by_msgctxt = by_msgctxt;
+        self
+    }
+
+    pub fn by_obsolete(mut self, by_obsolete: bool) -> Self {
+        self.by_obsolete = by_obsolete;
+        self
+    }
+
+    pub fn by_occurrences(mut self, by_occurrences: bool) -> Self {
+        self.by_occurrences = by_occurrences;
+        self
+    }
+
+    pub fn by_msgid_plural(mut self, by_msgid_plural: bool) -> Self {
+        self.by_msgid_plural = by_msgid_plural;
+        self
+    }
+
+    pub fn by_msgstr_plural(
+        mut self,
+        by_msgstr_plural: bool,
+    ) -> Self {
+        self.by_msgstr_plural = by_msgstr_plural;
+        self
+    }
+
+    pub fn by_flags(mut self, by_flags: bool) -> Self {
+        self.by_flags = by_flags;
+        self
+    }
+
+    pub fn by_all(mut self, by_all: bool) -> Self {
+        self.by_msgid = by_all;
+        self.by_msgstr = by_all;
+        self.by_msgctxt = by_all;
+        self.by_obsolete = by_all;
+        self.by_occurrences = by_all;
+        self.by_msgid_plural = by_all;
+        self.by_msgstr_plural = by_all;
+        self.by_flags = by_all;
+        self
+    }
+}
+
+impl Default for EntryCmpByOptions {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl From<&Vec<(String, bool)>> for EntryCmpByOptions {
+    fn from(options: &Vec<(String, bool)>) -> Self {
+        let mut ret = Self::new();
+        for (key, value) in options {
+            match key.as_str() {
+                "msgid" => ret.by_msgid = *value,
+                "msgstr" => ret.by_msgstr = *value,
+                "msgctxt" => ret.by_msgctxt = *value,
+                "obsolete" => ret.by_obsolete = *value,
+                "occurrences" => ret.by_occurrences = *value,
+                "msgid_plural" => ret.by_msgid_plural = *value,
+                "msgstr_plural" => ret.by_msgstr_plural = *value,
+                "flags" => ret.by_flags = *value,
+                _ => {}
+            }
+        }
+        ret
+    }
+}
